@@ -1,7 +1,7 @@
 import random, sys
 from cStringIO import StringIO
 from persistent import Persistent
-from zope import interface 
+from zope import interface
 from zope.event import notify
 from zope.security.management import getInteraction
 from zope.security.interfaces import  NoInteraction
@@ -13,7 +13,7 @@ try:
     from zope.annotation.interfaces import IAnnotations
 except:
     from zope.app.annotation.interfaces import IAnnotations
-    
+
 from zope.app.container.contained import Contained
 
 try:
@@ -21,7 +21,7 @@ try:
 except ImportError:
     from zope.app.event.objectevent import ObjectModifiedEvent
 
-try:    
+try:
     from zope.component.interfaces import ObjectEvent
 except ImportError:
     from zope.app.event.objectevent import ObjectEvent
@@ -41,8 +41,8 @@ def NullCondition(wf, context):
 def NullAction(wf, context):
     pass
 
-# XXX this is needed to make the tests pass in the absence of
-# interactions..
+# Used needed to make the tests pass in the absence of
+# interactions.. or when security checks are explicity not requested.
 def nullCheckPermission(permission, principal_id):
     return True
 
@@ -65,13 +65,13 @@ class Transition(object):
         self.permission = permission
         self.order = order
         self.user_data = user_data
-        
+
     def __cmp__(self, other):
         return cmp(self.order, other.order)
-    
+
 class Workflow(Persistent, Contained):
     interface.implements(IWorkflow)
-    
+
     def __init__(self, transitions):
         self.refresh(transitions)
 
@@ -86,13 +86,13 @@ class Workflow(Persistent, Contained):
         for transition in transitions:
             self._register(transition)
         self._p_changed = True
-        
+
     def getTransitions(self, source):
         try:
             return self._sources[source].values()
         except KeyError:
             return []
-        
+
     def getTransition(self, source, transition_id):
         transition = self._id_transitions[transition_id]
         if transition.source != source:
@@ -131,18 +131,16 @@ class Workflow(Persistent, Contained):
         fancier export workflow as dot,
 
         transition colors
-        
+
          - automatic triggers in green
          - system triggers in yellow if no conditions
          - system triggers in blue if conditional
          - manual triggers in red
 
         state colors
-        
+
          - end states in red
-         
         """
-        
         io = StringIO()
         states = set()
         end_states = set()
@@ -179,14 +177,14 @@ class Workflow(Persistent, Contained):
 
         return io.getvalue()
 
-        
+
 class WorkflowState(object):
-    
+
     interface.implements(IWorkflowState)
 
     workflow_state_key = "hurry.workflow.state"
     workflow_id_key  = "hurry.workflow.id"
-    
+
     def __init__(self, context):
         # XXX okay, I'm tired of it not being able to set annotations, so
         # we'll do this. Ugh.
@@ -196,15 +194,15 @@ class WorkflowState(object):
     def initialize(self):
         wf_versions = zapi.getUtility(IWorkflowVersions)
         self.setId(wf_versions.createVersionId())
-        
+
     def setState(self, state):
         if state != self.getState():
             IAnnotations(self.context)[ self.workflow_state_key ] = state
-            
+
     def setId(self, id):
         # XXX catalog should be informed (or should it?)
         IAnnotations(self.context)[ self.workflow_id_key ] = id
-        
+
     def getState(self):
         try:
             return IAnnotations(self.context)[ self.workflow_state_key ]
@@ -216,7 +214,7 @@ class WorkflowState(object):
             return IAnnotations(self.context)[ self.workflow_id_key ]
         except KeyError:
             return None
-            
+
 class WorkflowInfo(object):
 
     interface.implements(IWorkflowInfo)
@@ -233,7 +231,7 @@ class WorkflowInfo(object):
         if context is None:
             return IWorkflowState( self.context )
         return IWorkflowState( context )
-    
+
     def workflow( self ):
         return zapi.queryAdapter(self.context, IWorkflow) or zapi.getUtility(IWorkflow)
 
@@ -259,7 +257,7 @@ class WorkflowInfo(object):
 
     def _setState( self, state_id ):
         pass
-        
+
     def fireTransition(self, transition_id, comment=None, side_effect=None,
                        check_security=True):
         state = self.state()
@@ -269,7 +267,7 @@ class WorkflowInfo(object):
 
         self._check( transition, check_security )
         transition.action(self, self.context)
-        
+
         if transition.source is None:
             self.state().initialize()
             # execute any side effect
@@ -278,7 +276,7 @@ class WorkflowInfo(object):
 
         # change state of context or new object
         state.setState(transition.destination)
-        
+
         # allow workflow info to provide custom behavior for state changes
         self._setState(transition.destination)
 
@@ -286,20 +284,20 @@ class WorkflowInfo(object):
         event = WorkflowTransitionEvent(self.context, transition.source,
                                         transition.destination, transition, comment)
         notify(event)
-        
+
         # send modified event for original or new object
         notify(ObjectModifiedEvent(self.context))
 
     def fireTransitionToward(self, state, comment=None, side_effect=None,
                              check_security=True):
-        transition_ids = self.getFireableTransitionIdsToward(state)
+        transition_ids = self.getFireableTransitionIdsToward(state, check_security)
         if not transition_ids:
             raise interfaces.NoTransitionAvailableError
         if len(transition_ids) != 1:
             raise interfaces.AmbiguousTransitionError
         return self.fireTransition(transition_ids[0],
                                    comment, side_effect, check_security)
-        
+
     def fireTransitionForVersions(self, state, transition_id):
         id = self.state().getId()
         wf_versions = zapi.getUtility(IWorkflowVersions)
@@ -320,17 +318,20 @@ class WorkflowInfo(object):
                 # if we actually managed to fire a transition,
                 # we're done with this one now.
                 return
-            
+
     def hasVersion(self, state):
         wf_versions = zapi.getUtility(IWorkflowVersions)
         id = self.state().getId()
         return wf_versions.hasVersion(state, id)
-    
-    def getManualTransitionIds(self):
+
+    def getManualTransitionIds(self, check_security=True):
         try:
             checkPermission = getInteraction().checkPermission
         except NoInteraction:
             checkPermission = nullCheckPermission
+        if not check_security:
+            checkPermission = nullCheckPermission
+
         return [transition.transition_id for transition in
                 sorted(self._getTransitions(MANUAL)) if
                 transition.condition(self, self.context) and
@@ -342,9 +343,10 @@ class WorkflowInfo(object):
                 sorted(self._getTransitions(SYSTEM)) if
                 transition.condition(self, self.context)]
 
-    def getFireableTransitionIds(self):
-        return self.getManualTransitionIds() + self.getSystemTransitionIds()
-    
+    def getFireableTransitionIds(self, check_security=True):
+        return (self.getManualTransitionIds(check_security=check_security) +
+                self.getSystemTransitionIds())
+
     def getFireableTransitionIdsToward(self, state):
         wf = self.workflow()
         result = []
@@ -353,7 +355,7 @@ class WorkflowInfo(object):
             if transition.destination == state:
                 result.append(transition_id)
         return result
-    
+
     def getAutomaticTransitionIds(self):
         return [transition.transition_id for transition in
                 self._getTransitions(AUTOMATIC)]
@@ -373,14 +375,14 @@ class WorkflowInfo(object):
 
 class AdaptedWorkflowBase( object ):
     pass
-    
+
 def AdaptedWorkflow( workflow ):
     """
     wraps a IWorkflow utility into an adapter for use as an adapted workflow
     """
 
     assert IWorkflow.providedBy( workflow )
-    
+
     class AdaptedWorkflow( AdaptedWorkflowBase ):
 
         def __init__( self, context):
@@ -388,7 +390,7 @@ def AdaptedWorkflow( workflow ):
             self.workflow = workflow
 
             for m in IWorkflow.names():
-                setattr( self, m, getattr( workflow, m ) ) 
+                setattr( self, m, getattr( workflow, m ) )
 
     interface.classImplements( AdaptedWorkflow, IWorkflow )
     interface.directlyProvides( AdaptedWorkflow,
@@ -406,20 +408,20 @@ def ParallelWorkflow( workflow, wf_name, register_for=None):
     """
 
     assert IWorkflow.providedBy( workflow )
-    
+
     class _ParallelWorkflowState( WorkflowState ):
         workflow_state_key = "%s.state"%(wf_name)
         workflow_id_key = "%s.id"%(wf_name)
 
     class _ParallelWorkflowInfo( ParallelWorkflowInfo ):
         name = wf_name
-        
+
     if not register_for:
         return [ workflow, _ParallelWorkflowState, _ParallelWorkflowInfo ]
 
     # when you have a few these, zcml registration can be tedious, try to optionally
     # automate some of the pain, even if only for the global site manager
-    
+
     if IAdaptedWorkflow.providedBy( workflow ):
         component.provideAdapter( workflow, (register_for,), IWorkflow, wf_name )
     else:
@@ -429,7 +431,7 @@ def ParallelWorkflow( workflow, wf_name, register_for=None):
                               (register_for, ),
                               IWorkflowInfo,
                               wf_name )
-    
+
     component.provideAdapter( _ParallelWorkflowState,
                               (register_for, ),
                               IWorkflowState,
@@ -440,23 +442,21 @@ def ParallelWorkflow( workflow, wf_name, register_for=None):
 class ParallelWorkflowInfo( WorkflowInfo ):
 
     name = "ore_workflow"
-    
+
     def info( self, context = None ):
         if context is None:
             return zapi.getAdapter( self.context, IWorkflowInfo, self.name )
-        return zapi.getAdapter( context, IWorkflowInfo, self.name )        
+        return zapi.getAdapter( context, IWorkflowInfo, self.name )
 
     def state( self, context = None ):
         if context is None:
             return zapi.getAdapter( self.context, IWorkflowState, self.name )
         return zapi.getAdapter( context, IWorkflowState, self.name )
-    
+
     def workflow( self ):
         return zapi.queryAdapter( self.context, IWorkflow, self.name ) \
                or zapi.getUtility(IWorkflow, self.name )
 
-        
-                    
 class WorkflowVersions(object):
     interface.implements(IWorkflowVersions)
 
